import { StringParser } from '../primitive/string';
import { NumberParser } from '../primitive/number';
import { BooleanParser } from '../primitive/boolean';
import { CustomParser } from '../custom';
import { NullParser } from '../structure/null';
import { MutableArrayParser, ReadonlyArrayParser } from '../structure/array';
import { MutableObjectParser, ReadonlyObjectParser } from '../structure/object';
declare const parse: (value: unknown) => {
    asNull: () => NullParser;
    asString: () => StringParser;
    asNumber: () => NumberParser;
    asBoolean: () => BooleanParser;
    asCustom: <C>(predicate: ConstructorParameters<typeof CustomParser>[1]) => CustomParser<C>;
    asMutableArray: <E>(parseElement: (element: unknown, index?: number | undefined, array?: unknown[] | undefined) => E) => MutableArrayParser<E>;
    asReadonlyArray: <E_1>(parseElement: (element: unknown, index?: number | undefined, array?: unknown[] | undefined) => E_1) => ReadonlyArrayParser<E_1>;
    asMutableObject: <O extends Object>(parse: (element: unknown) => O) => MutableObjectParser<O>;
    asReadonlyObject: <O_1 extends Object>(parse: (element: unknown) => O_1) => ReadonlyObjectParser<O_1>;
};
declare class Parser {
    private readonly value;
    constructor(value: unknown);
    asString: () => StringParser;
    asNumber: () => NumberParser;
    asBoolean: () => BooleanParser;
    asNull: () => NullParser;
    asCustom: <C>(predicate: ConstructorParameters<typeof CustomParser>[1]) => CustomParser<C>;
    asMutableArray: <E>(parseElement: (element: unknown, index?: number | undefined, array?: unknown[] | undefined) => E) => MutableArrayParser<E>;
    asReadonlyArray: <E>(parseElement: (element: unknown, index?: number | undefined, array?: unknown[] | undefined) => E) => ReadonlyArrayParser<E>;
    asMutableObject: <O extends Object>(parse: (element: unknown) => O) => MutableObjectParser<O>;
    asReadonlyObject: <O extends Object>(parse: (element: unknown) => O) => ReadonlyObjectParser<O>;
}
declare const parser: (value: ConstructorParameters<typeof Parser>[0]) => Parser;
export default parse;
export { parser };
