import Parser from '../class/abstract';
import { type Get, type LazyGet, type Throw } from '../function/action';
type ArrayOptions<R> = Readonly<{
    array: unknown;
    parseElement: (element: unknown, index?: number, array?: Array<unknown>) => R;
}>;
declare function parseAsMutableArray<R, E extends Error>(options: Throw<E> & ArrayOptions<R>): Array<R>;
declare function parseAsMutableArray<R, T>(options: (Get<T> | LazyGet<T>) & ArrayOptions<R>): T | Array<R>;
declare function parseAsReadonlyArray<R, E extends Error>(options: Throw<E> & ArrayOptions<R>): ReadonlyArray<R>;
declare function parseAsReadonlyArray<R, T>(options: (Get<T> | LazyGet<T>) & ArrayOptions<R>): T | ReadonlyArray<R>;
type Parse<Element> = ArrayOptions<Element>['parseElement'];
declare abstract class ArrayParser<Element> extends Parser<ReadonlyArray<Element>> {
    protected readonly parseElement: Parse<Element>;
    constructor(value: unknown, parseElement: Parse<Element>);
}
declare class ReadonlyArrayParser<Element> extends ArrayParser<Element> {
    constructor(array: unknown, parseElement: Parse<Element>);
    elseGet: <A>(alternativeValue: A) => readonly Element[] | A;
    elseLazyGet: <A>(alternativeValue: () => A) => readonly Element[] | A;
    elseThrow: <E extends Error>(message: string | E) => ReadonlyArray<Element>;
}
declare class MutableArrayParser<Element> extends ArrayParser<Element> {
    constructor(array: unknown, parseElement: Parse<Element>);
    elseGet: <A>(alternativeValue: A) => Element[] | A;
    elseLazyGet: <A>(alternativeValue: () => A) => Element[] | A;
    elseThrow: <E extends Error>(message: string | E) => Array<Element>;
}
export { MutableArrayParser, parseAsMutableArray, ReadonlyArrayParser, parseAsReadonlyArray, };
